# Generated by Django 4.1.1 on 2023-01-20 21:33

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('django_form_generator', '0004_fieldoptionthrough_remove_fieldvaluethrough_field_and_more'),
    ]

    operations = [
        migrations.AlterField(
            model_name='field',
            name='genre',
            field=models.CharField(choices=[('text_input', 'Text input'), ('multi_text_input', 'Multi-Text input'), ('text_area', 'Text area'), ('number', 'Number'), ('dropdown', 'Dropdown'), ('date', 'Date'), ('time', 'Time'), ('datetime', 'Datetime'), ('email', 'E-Mail'), ('password', 'Password'), ('checkbox', 'Checkbox'), ('multi_checkbox', 'Multi checkbox'), ('radio', 'Radio'), ('hidden', 'Hidden'), ('captcha', 'Captcha'), ('upload_file', 'Upload file')], max_length=80, verbose_name='Genre'),
        ),
        migrations.AlterField(
            model_name='fieldvalidator',
            name='validator',
            field=models.CharField(choices=[('max-length', 'Max length'), ('min-length', 'Min length'), ('max-value', 'Max value'), ('min-value', 'Min value'), ('regex', 'Regex'), ('file-extention', 'File extention'), ('file-size', 'File size (MB)')], max_length=100, verbose_name='Validator'),
        ),
    ]



from django.urls import path, include
from django_form_generator.views import FormGeneratorView, FormResponseView


urlpatterns = [
    path('form/<int:pk>/', FormGeneratorView.as_view(), name="form_detail"),
    path('form-response/<uuid:unique_id>/', FormResponseView.as_view(), name="form_response"),
    path('api/', include(('django_form_generator.api.urls', 'django_form_generator'), 'api'))
]



import urllib
from django.contrib import admin
from django.urls import path
from django.utils.translation import gettext as _


class AdminMixin:

    extra_views: list[tuple[str, str]] = []
    "[('method_name', 'method/url/', 'url_name')] (url_name is optional it will be replaced with method_name if not provided)"

    def build_admin_url(self, url_name, reverse=False):
        prefix = ""
        if reverse:
            prefix = "admin:"
        return '%s%s_%s_%s' % (
            prefix,
            self.model._meta.app_label,
            self.model._meta.model_name,
            url_name
        )

    def get_extra_views(self):
        return self.extra_views

    def extra_view_context(self, request):
        opts = self.model._meta
        base_context = self.admin_site.each_context(request)
        base_context.update({
            "title": opts.verbose_name,
            "opts": opts,
            "module_name": str(opts.verbose_name_plural),
        })
        return base_context

    def get_urls(self):
        urls = super().get_urls()
        extra_views = list(map(lambda x: (x[0], x[1], x[2]) if len(x) == 3 else (x[0], x[1], x[0]), self.get_extra_views()))
        for view, url, name in extra_views:
            urls += [
                path(url, self.admin_site.admin_view(getattr(self, view)), name=self.build_admin_url(name))
            ]
        return urls


class FormFilter(admin.FieldListFilter):

    initial = {}
    form_class = None

    def __init__(self, field, request, params, model, model_admin, field_path):
        super().__init__(field, request, params, model, model_admin, field_path)
        self.request = request
        expected = self.expected_parameters()
        others = {k: v for k, v in request.GET.items() if k not in expected}
        self.other_query_string = urllib.parse.urlencode(others)
        self.form = self.get_form(request)
        if self.form:
            self.form.is_valid()

    def form_lookups(self):
        raise NotImplementedError(
            "subclasses of FormFieldFilter must provide a form_lookups() method"
        )

    def expected_parameters(self):
        return [item[0] for item in self.form_lookups()]

    def get_initial(self):
        return self.initial.copy()

    def get_form_kwargs(self, request):
        return {
            "prefix": self.field.name,
            "initial": self.get_initial(),
            "data": request.GET or None,
        }

    def get_form(self, request):
        return self.form_class(**self.get_form_kwargs(request))

    def get_lookups(self):
        lookups = {k.split("_")[1]: v for k, v in self.form_lookups()}
        data = self.form.cleaned_data if self.form.is_bound else {}
        return {v: data[k] for k, v in lookups.items() if data.get(k)}

    def queryset(self, request, queryset):
        lookup = self.get_lookups()
        if isinstance(lookup, dict):
            return queryset.filter(**lookup)
        return queryset.filter(lookup)

    def choices(self, changelist):
        return ()




# Generated by Django 4.0.6 on 2022-10-23 13:24

from django.db import migrations, models
import django.db.models.deletion
import uuid


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='Field',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True, verbose_name='Created at')),
                ('updated_at', models.DateTimeField(auto_now=True, verbose_name='Updated at')),
                ('label', models.CharField(max_length=100, verbose_name='Label')),
                ('name', models.CharField(blank=True, max_length=100, unique=True, verbose_name='Name')),
                ('genre', models.CharField(choices=[('text_input', 'Text input'), ('text_area', 'Text area'), ('number', 'Number'), ('dropdown', 'Dropdown'), ('date', 'Date'), ('time', 'Time'), ('datetime', 'Datetime'), ('email', 'E-Mail'), ('password', 'Password'), ('checkbox', 'Checkbox'), ('multi_checkbox', 'Multi checkbox'), ('radio', 'Radio'), ('hidden', 'Hidden'), ('captcha', 'Captcha'), ('upload_file', 'Upload file')], max_length=80, verbose_name='Genre')),
                ('is_required', models.BooleanField(default=True, verbose_name='Is Required')),
                ('placeholder', models.CharField(blank=True, max_length=100, null=True, verbose_name='Placeholder')),
                ('default', models.CharField(blank=True, max_length=255, null=True, verbose_name='Default Value')),
                ('help_text', models.CharField(blank=True, max_length=200, null=True, verbose_name='Help Text')),
                ('regex_pattern', models.CharField(blank=True, max_length=500, null=True, verbose_name='Regex Pattern')),
                ('error_message', models.CharField(blank=True, max_length=200, null=True, verbose_name='Error Message')),
                ('is_active', models.BooleanField(verbose_name='Is Active')),
                ('file_types', models.CharField(blank=True, help_text='example: jpg,png', max_length=100, null=True, verbose_name='Allow File Types')),
                ('file_size', models.IntegerField(blank=True, help_text='Default value will be set to 50 MB', null=True, verbose_name='File Size')),
                ('object_id', models.BigIntegerField(blank=True, null=True, verbose_name='Depends on Object ID')),
            ],
            options={
                'verbose_name': 'Field',
                'verbose_name_plural': 'Fields',
            },
        ),
        migrations.CreateModel(
            name='FieldCategory',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True, verbose_name='Created at')),
                ('updated_at', models.DateTimeField(auto_now=True, verbose_name='Updated at')),
                ('title', models.CharField(max_length=200, unique=True, verbose_name='Title')),
                ('is_active', models.BooleanField(default=True, verbose_name='Is Active')),
                ('weight', models.PositiveIntegerField(verbose_name='Weight')),
            ],
            options={
                'verbose_name': 'FieldCategory',
                'verbose_name_plural': 'FieldCategories',
                'ordering': ('weight',),
            },
        ),
        migrations.CreateModel(
            name='FieldValueThrough',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('weight', models.PositiveIntegerField(verbose_name='Weight')),
            ],
            options={
                'ordering': ('weight',),
            },
        ),
        migrations.CreateModel(
            name='Form',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True, verbose_name='Created at')),
                ('updated_at', models.DateTimeField(auto_now=True, verbose_name='Updated at')),
                ('title', models.CharField(max_length=200, verbose_name='Title')),
                ('slug', models.SlugField(allow_unicode=True, unique=True, verbose_name='Slug')),
                ('status', models.CharField(choices=[('draft', 'Draft'), ('pending', 'Pending'), ('publish', 'Publish'), ('suspend', 'Suspend')], default='draft', max_length=50, verbose_name='Status')),
                ('submit_text', models.CharField(default='Submit', max_length=100, verbose_name='Submit Text')),
                ('redirect_url', models.URLField(blank=True, null=True, verbose_name='Redirect URL')),
                ('success_message', models.CharField(blank=True, max_length=255, null=True, verbose_name='Success Message')),
                ('theme', models.CharField(blank=True, help_text='If choose dynamic_fields.html the order of fields depends on position of Field.', max_length=300, null=True, verbose_name='Theme')),
                ('direction', models.CharField(choices=[('ltr', 'LTR'), ('rtl', 'RTL')], default='ltr', max_length=3, verbose_name='Direction')),
                ('limit_to', models.PositiveIntegerField(blank=True, null=True, verbose_name='Limit Submiting Form')),
                ('valid_from', models.DateTimeField(blank=True, null=True, verbose_name='Valid From')),
                ('valid_to', models.DateTimeField(blank=True, null=True, verbose_name='Valid To')),
                ('is_editable', models.BooleanField(default=False, verbose_name='Is Editable')),
            ],
            options={
                'verbose_name': 'Form',
                'verbose_name_plural': 'Forms',
            },
        ),
        migrations.CreateModel(
            name='FormAPIManager',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True, verbose_name='Created at')),
                ('updated_at', models.DateTimeField(auto_now=True, verbose_name='Updated at')),
                ('title', models.CharField(max_length=200, verbose_name='Title')),
                ('url', models.URLField(verbose_name='URL')),
                ('headers', models.JSONField(blank=True, null=True, verbose_name='Headers')),
                ('method', models.CharField(choices=[('get', 'GET'), ('post', 'POST'), ('put', 'PUT'), ('patch', 'PATCH'), ('delete', 'DELETE'), ('option', 'OPTION')], max_length=50, verbose_name='Method')),
                ('body', models.TextField(blank=True, null=True, verbose_name='Body')),
                ('execute_time', models.CharField(choices=[('pre_load', 'Pre load'), ('post_load', 'Post load')], max_length=50, verbose_name='Execute Time')),
                ('response', models.TextField(blank=True, null=True, verbose_name='Response')),
                ('is_active', models.BooleanField(verbose_name='Is Active')),
            ],
            options={
                'verbose_name': 'FormAPIManager',
                'verbose_name_plural': 'FormAPIManagers',
            },
        ),
        migrations.CreateModel(
            name='FormAPIThrough',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('weight', models.PositiveIntegerField(verbose_name='Weight')),
            ],
            options={
                'ordering': ('weight',),
            },
        ),
        migrations.CreateModel(
            name='FormFieldThrough',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('position', models.CharField(choices=[('inline', 'In-line'), ('inorder', 'In-Order'), ('break', 'Break')], default='break', max_length=100, verbose_name='Position')),
                ('weight', models.PositiveIntegerField(verbose_name='Weight')),
            ],
            options={
                'ordering': ('weight',),
            },
        ),
        migrations.CreateModel(
            name='FormResponse',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True, verbose_name='Created at')),
                ('updated_at', models.DateTimeField(auto_now=True, verbose_name='Updated at')),
                ('unique_id', models.UUIDField(default=uuid.uuid4, unique=True, verbose_name='Unique ID')),
                ('data', models.JSONField(verbose_name='Data')),
                ('api_response', models.JSONField(blank=True, null=True, verbose_name='Api Respons')),
                ('user_ip', models.GenericIPAddressField(blank=True, null=True, verbose_name='IP Address')),
            ],
            options={
                'verbose_name': 'Form Response',
                'verbose_name_plural': 'Form Responses',
            },
        ),
        migrations.CreateModel(
            name='Value',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True, verbose_name='Created at')),
                ('updated_at', models.DateTimeField(auto_now=True, verbose_name='Updated at')),
                ('name', models.CharField(max_length=100, verbose_name='Name')),
                ('is_active', models.BooleanField(default=True, verbose_name='Is Active')),
            ],
            options={
                'verbose_name': 'Value',
                'verbose_name_plural': 'Values',
            },
        ),
        migrations.AddIndex(
            model_name='value',
            index=models.Index(fields=['name'], name='f_g_value_name'),
        ),
        migrations.AddField(
            model_name='formresponse',
            name='form',
            field=models.ForeignKey(on_delete=django.db.models.deletion.PROTECT, related_name='%(class)s_responses', to='django_form_generator.form', verbose_name='Form'),
        ),
        migrations.AddField(
            model_name='formfieldthrough',
            name='category',
            field=models.ForeignKey(blank=True, limit_choices_to={'is_active': True}, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='form_field_through', to='django_form_generator.fieldcategory', verbose_name='Category'),
        ),
        migrations.AddField(
            model_name='formfieldthrough',
            name='field',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='form_field_through', to='django_form_generator.field', verbose_name='Field'),
        ),
        migrations.AddField(
            model_name='formfieldthrough',
            name='form',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='form_field_through', to='django_form_generator.form', verbose_name='Form'),
        ),
        migrations.AddField(
            model_name='formapithrough',
            name='api',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='form_apis', to='django_form_generator.formapimanager', verbose_name='API'),
        ),
        migrations.AddField(
            model_name='formapithrough',
            name='form',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='form_apis', to='django_form_generator.form', verbose_name='Form'),
        ),
        migrations.AddIndex(
            model_name='formapimanager',
            index=models.Index(fields=['title'], name='f_g_formapimanager_title'),
        ),
        migrations.AddIndex(
            model_name='formapimanager',
            index=models.Index(fields=['method'], name='f_g_formapimanager_method'),
        ),
        migrations.AddIndex(
            model_name='formapimanager',
            index=models.Index(fields=['is_active'], name='f_g_formapimanager_active'),
        ),
        migrations.AddField(
            model_name='form',
            name='apis',
            field=models.ManyToManyField(related_name='forms', through='django_form_generator.FormAPIThrough', to='django_form_generator.formapimanager', verbose_name='Apis'),
        ),
        migrations.AddField(
            model_name='form',
            name='fields',
            field=models.ManyToManyField(related_name='forms', through='django_form_generator.FormFieldThrough', to='django_form_generator.field', verbose_name='Fields'),
        ),
        migrations.AddField(
            model_name='fieldvaluethrough',
            name='field',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='field_values', to='django_form_generator.field', verbose_name='Field'),
        ),
        migrations.AddField(
            model_name='fieldvaluethrough',
            name='value',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='field_values', to='django_form_generator.value', verbose_name='Value'),
        ),
        migrations.AddField(
            model_name='fieldcategory',
            name='parent',
            field=models.ForeignKey(blank=True, limit_choices_to={'is_active': True}, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='children', to='django_form_generator.fieldcategory', verbose_name='Parent Category'),
        ),
        migrations.AddField(
            model_name='field',
            name='content_type',
            field=models.ForeignKey(blank=True, limit_choices_to=models.Q(models.Q(('app_label', 'django_form_generator'), ('model', 'field')), models.Q(('app_label', 'django_form_generator'), ('model', 'value')), _connector='OR'), null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='fields', to='contenttypes.contenttype', verbose_name='Depends On Object'),
        ),
        migrations.AddField(
            model_name='field',
            name='values',
            field=models.ManyToManyField(help_text='Only for multi value fields like Dropdown, Radio, Checkbox, etc...', related_name='fields', through='django_form_generator.FieldValueThrough', to='django_form_generator.value', verbose_name='Values'),
        ),
        migrations.AddIndex(
            model_name='formresponse',
            index=models.Index(fields=['user_ip'], name='f_g_formresponse_user'),
        ),
        migrations.AddIndex(
            model_name='formresponse',
            index=models.Index(fields=['unique_id'], name='f_g_formresponse_unique_id'),
        ),
        migrations.AddIndex(
            model_name='formfieldthrough',
            index=models.Index(fields=['weight'], name='f_g_formfieldthrough_weight'),
        ),
        migrations.AddIndex(
            model_name='formapithrough',
            index=models.Index(fields=['weight'], name='f_g_formapithrough_weight'),
        ),
        migrations.AddIndex(
            model_name='form',
            index=models.Index(fields=['title'], name='f_g_form_title'),
        ),
        migrations.AddIndex(
            model_name='form',
            index=models.Index(fields=['slug'], name='f_g_form_slug'),
        ),
        migrations.AddIndex(
            model_name='form',
            index=models.Index(fields=['status'], name='f_g_form_status'),
        ),
        migrations.AddIndex(
            model_name='fieldvaluethrough',
            index=models.Index(fields=['weight'], name='f_g_fieldvaluethrough_weight'),
        ),
        migrations.AddIndex(
            model_name='fieldcategory',
            index=models.Index(fields=['title'], name='f_g_fieldcategory_title'),
        ),
        migrations.AddIndex(
            model_name='fieldcategory',
            index=models.Index(fields=['is_active'], name='f_g_fieldcategory_active'),
        ),
        migrations.AddIndex(
            model_name='fieldcategory',
            index=models.Index(fields=['weight'], name='f_g_fieldcategory_weight'),
        ),
        migrations.AddIndex(
            model_name='field',
            index=models.Index(fields=['label'], name='f_g_field_label'),
        ),
        migrations.AddIndex(
            model_name='field',
            index=models.Index(fields=['name'], name='f_g_field_name'),
        ),
        migrations.AddIndex(
            model_name='field',
            index=models.Index(fields=['genre'], name='f_g_field_genre'),
        ),
    ]


from django.db import models
from django.utils import timezone

from django_form_generator import const
from django_form_generator.settings import form_generator_settings as fg_settings

class FormQuerySet(models.QuerySet):

    def filter_valid(self) -> models.QuerySet:
        current_date = timezone.now()
        related_name = f'{fg_settings.FORM_GENERATOR_RESPONSE_MODEL._meta.model_name}_responses'
        lookup = self.annotate(c_forms=models.Count(related_name)).filter(status=const.FormStatus.PUBLISH).\
            filter(models.Q(limit_to__gt=models.F('c_forms')) | models.Q(limit_to__isnull=True)).\
            filter((models.Q(valid_from__lte=current_date) | models.Q(valid_from__isnull=True)) & 
                    (models.Q(valid_to__gte=current_date) | models.Q(valid_to__isnull=True)))
        return lookup


class FormManager(models.Manager):

    def get_queryset(self) -> FormQuerySet:
        return FormQuerySet(model=self.model, using=self._db)

    def filter_valid(self):
        return self.get_queryset().filter_valid()


# Generated by Django 4.0.6 on 2023-01-04 06:54

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('django_form_generator', '0002_field_read_only_field_write_only'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='field',
            name='error_message',
        ),
        migrations.RemoveField(
            model_name='field',
            name='file_size',
        ),
        migrations.RemoveField(
            model_name='field',
            name='file_types',
        ),
        migrations.RemoveField(
            model_name='field',
            name='regex_pattern',
        ),
        migrations.CreateModel(
            name='FieldValidator',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True, verbose_name='Created at')),
                ('updated_at', models.DateTimeField(auto_now=True, verbose_name='Updated at')),
                ('validator', models.CharField(choices=[('max-length', 'Max length'), ('min-length', 'Min length'), ('max-value', 'Max value'), ('min-value', 'Min value'), ('regex', 'Regex'), ('file-extention', 'File extention'), ('file-size', 'File size')], max_length=100, verbose_name='Validator')),
                ('value', models.CharField(max_length=255, verbose_name='Value')),
                ('error_message', models.CharField(blank=True, max_length=500, null=True, verbose_name='Error Message')),
                ('is_active', models.BooleanField(default=True, verbose_name='Is Active')),
                ('field', models.ForeignKey(on_delete=django.db.models.deletion.PROTECT, related_name='validators', to='django_form_generator.field', verbose_name='Field')),
            ],
            options={
                'unique_together': {('field', 'validator')},
            },
        ),
    ]



from django import forms
from django.utils.translation import gettext as _

from captcha.fields import ReCaptchaField
from captcha.widgets import ReCaptchaV2Checkbox
from tempus_dominus.widgets import DatePicker, TimePicker, DateTimePicker

from django_form_generator.settings import form_generator_settings as fg_settings
from django_form_generator.models import Field, Form, Option, FieldValidator
from django_form_generator.fields import MultiInputWidgetField, MultiInputField, CustomeSelectFormField
from django_form_generator import const


class FormGeneratorBaseForm(forms.Form):
    def __init__(self, form, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.instance = form
        self.template_name_p = getattr(self.instance, "style", self.template_name_p)
        self._initial_fields()
        
    def _initial_fields(self):
        for field in self.instance.get_fields():
            method = f"prepare_{field.genre}"
            if hasattr(self, method):
                self.fields[field.name] = getattr(self, method)(self.instance, field)
                self._handel_required_fields(field, self.fields[field.name])

    def _handel_required_fields(self, field, form_field):
        if self.data and field.content_object:
            if isinstance(field.content_object, Option):
                field_name = self.instance.get_fields(extra={"id__in":
                                                field.content_object.fields.values_list('id', flat=True)}).last().name
                parent_data = self.data.get(field_name, '')
                if str(field.object_id) not in parent_data:
                    form_field.required = False
                else:
                    form_field.widget.attrs.update({'disabled': False})
            else:
                parent_data = self.data.get(field.content_object.name, '')
                if len(parent_data) <= 0:
                    form_field.required = False
                else:
                    form_field.widget.attrs.update({'disabled': False})

    def prepare_text_input(self, form: Form, field: Field):
        widget_attrs: dict = field.build_widget_attrs(
            form, {"content_type": "field"}
        )
        field_attrs: dict = field.build_field_attrs(
            {"widget": forms.TextInput(attrs=widget_attrs)}
        )
        return forms.CharField(**field_attrs)

    def prepare_multi_text_input(self, form: Form, field: Field):
        widget_attrs: dict = field.build_widget_attrs(
            form, {"content_type": "field", "multi-input": True}
        )
        field_attrs: dict = field.build_field_attrs(
            {"widget": MultiInputWidgetField(attrs=widget_attrs)}
        )
        return MultiInputField(**field_attrs)

    def prepare_text_area(self, form: Form, field: Field):
        widget_attrs: dict = field.build_widget_attrs(
            form, {"content_type": "field"}
        )
        field_attrs: dict = field.build_field_attrs(
            {"widget": forms.Textarea(attrs=widget_attrs)}
        )
        return forms.CharField(**field_attrs)

    def prepare_number(self, form: Form, field: Field):
        widget_attrs: dict = field.build_widget_attrs(
            form, {"content_type": "field"}
        )
        field_attrs: dict = field.build_field_attrs(
            {"widget": forms.NumberInput(attrs=widget_attrs)}
        )
        return forms.IntegerField(**field_attrs)

    def prepare_date(self, form: Form, field: Field):
        widget_attrs: dict = field.build_widget_attrs(
            form, {"content_type": "field"}
        )
        field_attrs: dict = field.build_field_attrs(
            {"widget": DatePicker(attrs=widget_attrs)}
        )
        return forms.DateField(**field_attrs)

    def prepare_time(self, form: Form, field: Field):
        widget_attrs: dict = field.build_widget_attrs(
            form, {"content_type": "field"}
        )
        field_attrs: dict = field.build_field_attrs(
            {"widget": TimePicker(attrs=widget_attrs)}
        )
        return forms.TimeField(**field_attrs)

    def prepare_datetime(self, form: Form, field: Field):
        widget_attrs: dict = field.build_widget_attrs(
            form, {"content_type": "field"}
        )
        field_attrs: dict = field.build_field_attrs(
            {"widget": DateTimePicker(attrs=widget_attrs)}
        )
        return forms.DateTimeField(**field_attrs)

    def prepare_email(self, form: Form, field: Field):
        widget_attrs: dict = field.build_widget_attrs(
            form, {"content_type": "field"}
        )
        field_attrs: dict = field.build_field_attrs(
            {"widget": forms.EmailInput(attrs=widget_attrs)}
        )
        return forms.EmailField(**field_attrs)

    def prepare_password(self, form: Form, field: Field):
        widget_attrs: dict = field.build_widget_attrs(
            form, {"content_type": "field"}
        )
        field_attrs: dict = field.build_field_attrs(
            {"widget": forms.PasswordInput(attrs=widget_attrs)}
        )
        return forms.CharField(**field_attrs)

    def prepare_checkbox(self, form: Form, field: Field):
        widget_attrs: dict = field.build_widget_attrs(
            form, {"content_type": "field"}
        )
        field_attrs: dict = field.build_field_attrs(
            {"widget": forms.CheckboxInput(attrs=widget_attrs)}
        )
        return forms.BooleanField(**field_attrs)

    def prepare_dropdown(self, form: Form, field: Field):
        widget_attrs: dict = field.build_widget_attrs(
            form, {"content_type": "option"}
        )
        choices = field.get_choices().values_list("id", "name")
        field_attrs: dict = field.build_field_attrs(
            {
                "widget": CustomeSelectFormField(attrs=widget_attrs),
                "choices": tuple(choices),
            }
        )
        return forms.ChoiceField(**field_attrs)

    def prepare_multi_checkbox(self, form: Form, field: Field):
        widget_attrs: dict = field.build_widget_attrs(
            form, {"content_type": "option"}
        )
        choices = field.get_choices().values_list("id", "name")
        field_attrs: dict = field.build_field_attrs(
            {
                "widget": forms.CheckboxSelectMultiple(attrs=widget_attrs),
                "choices": tuple(choices),
            }
        )
        return forms.MultipleChoiceField(**field_attrs)

    def prepare_radio(self, form: Form, field: Field):
        widget_attrs: dict = field.build_widget_attrs(
            form, {"content_type": "option"}
        )
        choices = field.get_choices().values_list("id", "name")
        field_attrs: dict = field.build_field_attrs(
            {"widget": forms.RadioSelect(attrs=widget_attrs), "choices": tuple(choices)}
        )
        return forms.ChoiceField(**field_attrs)

    def prepare_hidden(self, form: Form, field: Field):
        widget_attrs: dict = field.build_widget_attrs(
            form, {"content_type": "field"}
        )
        field_attrs: dict = field.build_field_attrs(
            {"widget": forms.HiddenInput(attrs=widget_attrs)}
        )
        return forms.CharField(**field_attrs)

    def prepare_captcha(self, form: Form, field: Field):
        widget_attrs: dict = field.build_widget_attrs(
            form, {"content_type": "field"}
        )
        field_attrs: dict = field.build_field_attrs(
            {"widget": ReCaptchaV2Checkbox(attrs=widget_attrs)}
        )
        return ReCaptchaField(**field_attrs)

    def prepare_upload_file(self, form: Form, field: Field):
        widget_attrs: dict = field.build_widget_attrs(
            form, {"multiple": True, "content_type": "field"}
        )
        field_attrs: dict = field.build_field_attrs(
            {
                "widget": forms.ClearableFileInput(attrs=widget_attrs),
            }
        )
        return forms.FileField(**field_attrs)


class FormGeneratorForm(FormGeneratorBaseForm):
    def __init__(self, form, request, user_ip, *args, **kwargs):
        self.user_ip = user_ip
        self.request = request
        super().__init__(form, *args, **kwargs)

    def save(self):
        form_data = self.cleaned_data.copy()
        form_data.setdefault("request", self.request)
        save_module = fg_settings.FORM_RESPONSE_SAVE  
        save_module(self.instance, form_data, self.user_ip)# type: ignore


class FormGeneratorResponseForm(FormGeneratorBaseForm):
    def __init__(self, form, request, form_response, *args, **kwargs):
        self.request = request
        self.form_response = form_response
        super().__init__(form, *args, **kwargs)

    def _initial_fields(self):
        form_response_data = self.form_response.get_data()
        for i, field in enumerate(self.form_response.form.get_fields()):
            field_name = field.name
            method = f"prepare_{field.genre}"
            if hasattr(self, method):
                self.fields[field_name] = getattr(self, method)(self.instance, field)
                try:
                    initial_value = form_response_data[i].get(
                        "value", None
                    )
                    if (not initial_value and isinstance(initial_value, (list, tuple))) or field.write_only:
                        initial_value = None
                    self.fields[field_name].initial = initial_value
                    if initial_value:
                        try:
                            del self.fields[field_name].widget.attrs["disabled"]
                        except KeyError:
                            pass
                except IndexError:
                    pass
                
                if not self.instance.is_editable:
                    self.fields[field_name].widget.attrs.update({"disabled": True})
                
                self._handel_required_fields(field, self.fields[field.name])

    def save(self):
        save_module = fg_settings.FORM_RESPONSE_SAVE
        form_data = self.form_response.pure_data
        for changed_data in self.changed_data:
            form_data[changed_data] = self.cleaned_data[changed_data]

        form_data.setdefault("request", self.request)
        save_module(self.instance, form_data, update_form_response_id=self.form_response.id)# type: ignore
        


class FieldForm(forms.ModelForm):
    class Meta:
        model = Field
        fields = "__all__"

    def clean(self):
        cleaned_data = super().clean()
        if 'genre' in cleaned_data and cleaned_data["genre"] == const.FieldGenre.UPLOAD_FILE:
            data = [val for key, val in self.data.items() if key.startswith('validators') and key.endswith('validator') and val]
            if const.Validator.FILE_EXTENTION.value not in data or const.Validator.FILE_SIZE.value not in data:
                raise forms.ValidationError(
                    _("You should define FileExtention and FileSize validators for upload file genre")
                )
        return cleaned_data

class FormAdminForm(forms.ModelForm):
    style = forms.ChoiceField(choices=fg_settings.FORM_STYLE_CHOICES.choices)  # type: ignore

    class Meta:
        model = Form
        fields = "__all__"


class ValidatorAdminForm(forms.ModelForm):

    class Meta:
        model = FieldValidator
        fields = '__all__'

    def clean_value(self):
        value = self.cleaned_data['value']
        if value:
            const.Validator(self.cleaned_data['validator']).clean(value)
        return value

class FormResponseFilterForm(forms.Form):
    operand = forms.ChoiceField(choices=(('OR', 'OR'), ('AND', 'AND')),  required=False)
    field = forms.ModelChoiceField(Field.objects.all(), required=False, widget=forms.widgets.Select(attrs={'onchange': 'valueField(event)'}))
    text = forms.CharField(max_length=120, required=False)


from rest_framework.response import Response
from rest_framework import status

from django_form_generator.common.utils import get_client_ip
from django_form_generator.common.views import BaseAPIView
from django_form_generator.api.serializers import FormGeneratorResponseSerializer, FormGeneratorSerializer, FormSerializer, FormFullSerializer
from django_form_generator.models import Form
from django_form_generator.settings import form_generator_settings as fg_settings



class FormAPIView(BaseAPIView):
    serializer_class = FormSerializer
    queryset = Form.objects.filter_valid()
    model = Form

    def get(self, request, format=None):
        serializer_class = self.get_serializer_class(request)
        instance = self.get_queryset(request)
        serializer = serializer_class(instance, many=True)
        return Response(serializer.data)


class FormGeneratorAPIView(BaseAPIView):
    serializer_class = FormGeneratorSerializer
    queryset = Form.objects.filter_valid()
    model = Form
    lookup_field = 'pk'

    def get_serializer_class(self, request):
        return fg_settings.FORM_GENERATOR_SERIALIZER

    def get(self, request, pk, format=None):
        instance = self.get_object(pk)
        instance.call_pre_apis({"request": request})
        serializer = FormFullSerializer(instance=instance)
        return Response(serializer.data)

    def post(self, request, *args, **kwargs):
        serializer_class = self.get_serializer_class(request)
        instance = self.get_object(kwargs['pk'])
        serializer = serializer_class(data=request.data, context={'request': request, 'form': instance, 'user_ip': get_client_ip(request)})
        if serializer.is_valid():
            serializer.save()
            return Response(serializer.data, status=status.HTTP_201_CREATED)
        return Response(serializer.errors, status=status.HTTP_400_BAD_REQUEST)


class FormGeneratorResponseAPIView(BaseAPIView):
    serializer_class = FormGeneratorResponseSerializer
    queryset = fg_settings.FORM_GENERATOR_RESPONSE_MODEL.objects.all() #type: ignore
    model = fg_settings.FORM_GENERATOR_RESPONSE_MODEL
    lookup_field = 'unique_id'

    def get_serializer_class(self, request):
        return fg_settings.FORM_RESPONSE_SERIALIZER

    def get(self, request, unique_id, format=None):
        serializer_class = self.get_serializer_class(request)
        instance = self.get_object(unique_id)
        serializer = serializer_class(instance, context={'request': request, 'form': instance.form, 'form_response': instance})
        return Response(serializer.output_data)

    def patch(self, request, unique_id, format=None):
        serializer_class = self.get_serializer_class(request)
        instance = self.get_object(unique_id)
        serializer = serializer_class(instance.pure_data, data=request.data, partial=True, 
                context={'request': request, 'form': instance.form, 'form_response': instance})
        if serializer.is_valid():
            serializer.save()
            return Response(serializer.data)
        return Response(serializer.errors, status=status.HTTP_400_BAD_REQUEST)



from django import template
from django.urls import reverse
from django.utils.safestring import mark_safe

from django_form_generator.models import Form, Form


register = template.Library()


@register.inclusion_tag('django_form_generator/tags/form_tag.html')
def render_form(form_id: int):
    return {'url': reverse('django_form_generator:form_detail', args=(form_id,))}


@register.simple_tag(takes_context=True)
def render_pre_api(context, form_id, api_id=None):
    try:
        form = Form.objects.filter_valid().get(id=form_id)
    except Form.DoesNotExist:
        return 'Form id is not valid'
    else:
        responses = form.render_pre_apis({'request': context['request']})
        if api_id:
            responses = mark_safe(next((result for api_id_, result in responses if api_id_ == api_id)))
        else:
            responses = mark_safe('<br>'.join([result for _, result in responses]))
        return responses


@register.simple_tag(takes_context=True)
def render_post_api(context, form_id, api_id=None):
    try:
        form = Form.objects.filter_valid().get(id=form_id)
    except Form.DoesNotExist:
        return 'Form id is not valid'
    else:
        responses = form.render_post_apis({'request': context['request']})
        if api_id:
            responses = mark_safe(next((result for api_id_, result in responses if api_id_ == api_id)))
        else:
            responses = mark_safe('<br>'.join([result for _, result in responses]))
        return responses


@register.filter(takes_context=True)
def eval_data(context, body):
    tmp = template.Template(body)
    context = template.Context(context)
    data = tmp.render(context)
    return data


from django.test import TestCase

# Create your tests here.


class TestFormGenerator(TestCase):


    def test_get_published_forms(self):
        ...

    def test_get_non_published_forms(self):
        ...

    def test_submit_form_successfully(self):
        ...

    def test_submit_form_failed(self):
        ...

    def test_generate_fields(self):
        ...

    def test_pre_api(self):
        ...

    def test_render_pre_api(self):
        ...

    def test_post_api(self):
        ...

    def test_render_post_api(self):
        ...

    def test_edit_form_successfully(self):
        ...

    def test_edit_form_failed(self):
        ...

    def test_show_response(self):
        ...



    


from django.urls import path
from rest_framework.urlpatterns import format_suffix_patterns
from django_form_generator.api import views

urlpatterns = [
    path('forms/', views.FormAPIView.as_view(), name="api_forms"),
    path('forms/<int:pk>/', views.FormGeneratorAPIView.as_view(), name="api_form_detail"),
    path('form-response/<uuid:unique_id>/', views.FormGeneratorResponseAPIView.as_view(), name="api_form_response"),
]

urlpatterns=format_suffix_patterns(urlpatterns)


from django import forms


class MultiInputWidgetField(forms.widgets.TextInput):
    allow_multiple_selected = True

    def format_value(self, value):
        if not value:
            return None
        if isinstance(value, (list, tuple)):
            return value

    def value_from_datadict(self, data, files, name):
        getter = data.get
        if self.allow_multiple_selected:
            try:
                getter = data.getlist
            except AttributeError:
                pass
        return getter(name)

    def get_context(self, name, value, attrs):
        context = super().get_context(name, value, attrs)
        if self.allow_multiple_selected:
            context["widget"]["attrs"]["multiple"] = True
        return context

class MultiInputField(forms.CharField):

    def to_python(self, value):
        if not value:
            return []
        elif not isinstance(value, (list, tuple)):
            raise forms.ValidationError(
                self.error_messages["invalid_list"], code="invalid_list"
            )
        return [str(val) for val in value if val]


class CustomeSelectFormField(forms.Select):
    option_inherits_attrs = True


from django.views.generic import DetailView
from django.views.generic.edit import FormMixin
from django.contrib import messages
from django.utils.translation import gettext as _

from django_htmx.http import HttpResponseClientRedirect

from django_form_generator.common.utils import get_client_ip
from django_form_generator.models import Form
from django_form_generator.forms import FormGeneratorForm
from django_form_generator.settings import form_generator_settings as fg_settings


class FormGeneratorView(FormMixin, DetailView):
    queryset = Form.objects.filter_valid()
    model = Form
    template_name = "django_form_generator/form.html"

    def get_queryset(self):
        """Allow preview for superuser and staff members"""
        if getattr(self.request.user, "is_staff", False) or getattr(self.request.user, "is_superuser", False):
            return Form.objects.all()
        return super().get_queryset()

    def get_form_class(self):
        return fg_settings.FORM_GENERATOR_FORM

    def get_success_url(self) -> str:
        return self.object.redirect_url or self.request.META.get("HTTP_REFERER")  # type: ignore

    def post(self, request, *args, **kwargs):
        self.object = self.get_object()
        form = self.get_form()
        if form.is_valid():
            messages.success(
                self.request,
                self.object.success_message or _("Form submited successfully."),
                "success",
            )
            return self.form_valid(form)
        else:
            messages.success(self.request, str(form.errors), "danger")
            return self.form_invalid(form)

    def get(self, request, *args, **kwargs):
        response = super().get(request, *args, **kwargs)
        self.object.call_pre_apis({"request": request})
        return response

    def get_form_kwargs(self):
        kwargs = super().get_form_kwargs()
        kwargs.update(
            {
                "request": self.request,
                "form": self.object,
                "user_ip": get_client_ip(self.request),
            }
        )
        return kwargs

    def form_valid(self, form: FormGeneratorForm):
        form.save()
        return HttpResponseClientRedirect(self.get_success_url())


class FormResponseView(FormMixin, DetailView):
    queryset = fg_settings.FORM_GENERATOR_RESPONSE_MODEL.objects.all()
    model = fg_settings.FORM_GENERATOR_RESPONSE_MODEL
    template_name = 'django_form_generator/form_response.html'
    slug_url_kwarg = 'unique_id'
    slug_field = 'unique_id'


    def get_form_class(self):
        return fg_settings.FORM_RESPONSE_FORM

    def get_success_url(self) -> str:
        return self.object.form.redirect_url or self.request.META.get("HTTP_REFERER")  # type: ignore

    def post(self, request, *args, **kwargs):
        self.object = self.get_object()
        form = self.get_form()
        if form.is_valid():
            messages.success(
                self.request,
                self.object.form.success_message or _("Form submited successfully."),
                "success",
            )
            return self.form_valid(form)
        else:
            messages.success(self.request, str(form.errors), "danger")
            return self.form_invalid(form)

    def form_valid(self, form):
        form.save()
        return super().form_valid(form)

    def get_form_kwargs(self):
        kwargs = super().get_form_kwargs()
        kwargs.update(
            {
                "form": self.object.form,
                "form_response": self.object,
                "request": self.request
            }
        )
        return kwargs



# Generated by Django 4.1.1 on 2022-12-18 07:26

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('django_form_generator', '0001_initial'),
    ]

    operations = [
            migrations.AddField(
                model_name='field',
                name='read_only',
                field=models.BooleanField(default=False, verbose_name='Read Only'),
            ),
            migrations.AddField(
                model_name='field',
                name='write_only',
                field=models.BooleanField(default=False, verbose_name='Write Only'),
            ),
    ]



from django.db.models import TextChoices
from django.utils.translation import gettext_lazy as _
from django.core.exceptions import ValidationError
from django.core.validators import (MaxValueValidator, 
                                    MinValueValidator, 
                                    MaxLengthValidator, 
                                    MinLengthValidator, 
                                    RegexValidator,
                                    FileExtensionValidator,
                                    )
from django.utils.module_loading import import_string


class FormStatus(TextChoices):
    DRAFT = 'draft', _('Draft')
    PENDING = 'pending', _('Pending')
    PUBLISH = 'publish', _('Publish')
    SUSPEND = 'suspend', _('Suspend')

class FieldGenre(TextChoices):
    TEXT_INPUT = 'text_input', _('Text input')
    MULTI_TEXT_INPUT = 'multi_text_input', _('Multi-Text input')
    TEXT_AREA = 'text_area', _('Text area')
    NUMBER = 'number', _('Number')
    DROPDOWN = 'dropdown', _('Dropdown')
    DATE = 'date', _('Date')
    TIME = 'time', _('Time')
    DATETIME = 'datetime', _('Datetime')
    EMAIL = 'email', _('E-Mail')
    PASSWORD = 'password', _('Password')
    CHECKBOX = 'checkbox', _('Checkbox')
    MULTI_CHECKBOX = 'multi_checkbox', _('Multi checkbox')
    RADIO = 'radio', _('Radio')
    HIDDEN = 'hidden', _('Hidden')
    CAPTCHA = 'captcha', _('Captcha')
    UPLOAD_FILE = 'upload_file', _('Upload file')

    @classmethod
    def selectable_fields(cls):
        return [cls.DROPDOWN, cls.CHECKBOX, cls.RADIO, cls.MULTI_CHECKBOX]

    def evaluate(self, value, **kwargs):
        return getattr(self, "eval_" + self.name.lower())(value, **kwargs)

    def eval_text_input(self, value, **kwargs):
        if value is None or value == '':
            return None
        return str(value)

    def eval_multi_text_input(self, value, **kwargs):
        if value is None or value == '':
            return []
        if isinstance(value, list):
            return [str(val) for val in value]
        else:
            return []

    def eval_text_area(self, value, **kwargs):
        if value is None or value == '':
            return value
        return self.eval_text_input(value)

    def eval_number(self, value, **kwargs):
        if value is None or value == '':
            return None
        return int(value)

    def eval_dropdown(self, value, **kwargs):
        return self.eval_number(value)

    def eval_date(self, value, **kwargs):
        return self.eval_text_input(value)

    def eval_time(self, value, **kwargs):
        return self.eval_text_input(value)

    def eval_datetime(self, value, **kwargs):
        return self.eval_text_input(value)

    def eval_email(self, value, **kwargs):
        return self.eval_text_input(value)

    def eval_password(self, value, **kwargs):
        return self.eval_text_input(value)

    def eval_checkbox(self, value, **kwargs):
        return bool(value)

    def eval_multi_checkbox(self, value, **kwargs):
        if value is None or value == '':
            return []
        if isinstance(value, list):
            return [int(val) for val in value]
        else:
            return []

    def eval_radio(self, value, **kwargs):
        return self.eval_number(value)

    def eval_hidden(self, value, **kwargs):
        return self.eval_text_input(value)

    def eval_captcha(self, value, **kwargs):
        return None

    def eval_upload_file(self, value, **kwargs):
        FileFieldHelper = import_string('django_form_generator.common.helpers.FileFieldHelper')

        if (
            value is not None
            and not isinstance(value, dict)
        ):
            return FileFieldHelper.upload_file(
                kwargs['host'], value, kwargs['instance_directory']
            )
        return  value


class FormAPIManagerMethod(TextChoices):
    GET = 'get', _('GET')
    POST = 'post', _('POST')
    PUT = 'put', _('PUT')
    PATCH = 'patch', _('PATCH')
    DELETE = 'delete', _('DELETE')
    OPTION = 'option', _('OPTION')

class FormAPIManagerExecuteTime(TextChoices):
    PRE_LOAD = 'pre_load', _('Pre load')
    POST_LOAD = 'post_load', _('Post load')

class FieldPosition(TextChoices):
    INLINE = 'inline', _('In-line')
    INORDER = 'inorder', _('In-Order')
    BREAK = 'break', _('Break')

class FormDirection(TextChoices):
    LTR = 'ltr', _('LTR')
    RTL = 'rtl', _('RTL')

class FormStyle(TextChoices):
    INLINE = 'django_form_generator/fields/inline_fields.html', _('In-line')
    INORDER = 'django_form_generator/fields/inorder_fields.html', _('In-order')
    DYNAMIC = 'django_form_generator/fields/dynamic_fields.html', _('Dynamic')


class Validator(TextChoices):
    MAX_LENGTH = 'max-length', _('Max length')
    MIN_LENGTH = 'min-length', _('Min length')
    MAX_VALUE = 'max-value', _('Max value')
    MIN_VALUE = 'min-value', _('Min value')
    REGEX = 'regex', _('Regex')
    FILE_EXTENTION = 'file-extention', _('File extention')
    FILE_SIZE = 'file-size', _('File size (MB)')

    @classmethod
    def __validators_map(cls):
        FileSizeValidator = import_string('django_form_generator.common.utils.FileSizeValidator')

        validators_map = {
        cls.MAX_LENGTH: MaxLengthValidator,
        cls.MIN_LENGTH: MinLengthValidator,
        cls.MAX_VALUE: MaxValueValidator,
        cls.MIN_VALUE: MinValueValidator,
        cls.REGEX: RegexValidator,
        cls.FILE_EXTENTION: FileExtensionValidator,
        cls.FILE_SIZE: FileSizeValidator,
        }
        return validators_map

    def clean(self, value):
        getattr(self, "clean_" + self.name.lower())(value)

    def evaluate(self, value):
        return getattr(self, "eval_" + self.name.lower())(value)

    def eval_max_length(self, value):
        return int(value)

    def eval_min_length(self, value):
        return int(value)

    def eval_max_value(self, value):
        return int(value)

    def eval_min_value(self, value):
        return int(value)

    def eval_file_extention(self, value):
        return value.split(',')

    def eval_file_size(self, value):
        return float(value) * 2 ** 20

    def clean_max_length(self, value):
        try:
            int(value)
        except ValueError:
            raise ValidationError(_("Value for %s should be integer") % self.label)

    def clean_min_length(self, value):
        self.clean_max_length(value)

    def clean_max_value(self, value):
        self.clean_max_length(value)

    def clean_min_value(self, value):
        self.clean_max_length(value)

    def clean_file_extention(self, value):
        if ',' not in value and len(value.split(' ')) > 1:
            raise ValidationError(_('Separate values with comma for %s like: jpg,png,...') % self.label)

    def clean_file_size(self, value):
        try:
            float(value)
        except ValueError:
            raise ValidationError(_("Value for %s should be integer or float") % self.label)

    def validate(self, value, error_message=None):
        value = self.evaluate(value)
        return self.__validators_map()[self](value, error_message)


class CacheMethod(TextChoices):
    SESSION_KEY = 'session_key', _('Session Key')
    USER_ID = 'user_id', _('User ID')
    USER_IP = 'user_ip', _('User IP')


from rest_framework.views import APIView
from django.http import Http404


class BaseAPIView(APIView):
    serializer_class = ...
    queryset = ...
    model = ...
    lookup_field = ...

    def get_queryset(self, request=None):
        return self.queryset.all()
    
    def get_serializer_class(self, request):
        return self.serializer_class
    
    def get_object(self, arg):
        try:
            return self.get_queryset().get(**{self.lookup_field: arg})
        except self.model.DoesNotExist:
            raise Http404


# Generated by Django 4.1.1 on 2023-01-17 15:15

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('contenttypes', '0002_remove_content_type_name'),
        ('django_form_generator', '0003_remove_field_error_message_remove_field_file_size_and_more'),
    ]

    operations = [
        migrations.CreateModel(
            name='FieldOptionThrough',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('weight', models.PositiveIntegerField(verbose_name='Weight')),
            ],
            options={
                'ordering': ('weight',),
            },
        ),
        migrations.RenameModel(
            old_name='Value',
            new_name='Option',
        ),
        migrations.RemoveField(
            model_name='fieldvaluethrough',
            name='field',
        ),
        migrations.RemoveField(
            model_name='fieldvaluethrough',
            name='value',
        ),
        migrations.AlterModelOptions(
            name='option',
            options={'verbose_name': 'Option', 'verbose_name_plural': 'Options'},
        ),
        migrations.RemoveIndex(
            model_name='option',
            name='f_g_value_name',
        ),
        migrations.AddIndex(
            model_name='option',
            index=models.Index(fields=['name'], name='f_g_option_name'),
        ),
        migrations.RemoveField(
            model_name='field',
            name='values',
        ),
        migrations.RemoveField(
            model_name='form',
            name='theme',
        ),
        migrations.AddField(
            model_name='form',
            name='style',
            field=models.CharField(blank=True, help_text='Inline: 2 field in every row.Inorder: 1 field in every row.dynamic: can dynamicly style the field in FormFieldThrough section/table.', max_length=300, null=True, verbose_name='Style'),
        ),
        migrations.AddField(
            model_name='formapimanager',
            name='cache_by',
            field=models.CharField(blank=True, choices=[('session_key', 'Session Key'), ('user_id', 'User ID'), ('user_ip', 'User IP')], max_length=15, null=True, verbose_name='Cache By'),
        ),
        migrations.AlterField(
            model_name='field',
            name='content_type',
            field=models.ForeignKey(blank=True, limit_choices_to=models.Q(models.Q(('app_label', 'django_form_generator'), ('model', 'field')), models.Q(('app_label', 'django_form_generator'), ('model', 'option')), _connector='OR'), null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='fields', to='contenttypes.contenttype', verbose_name='Depends On Object'),
        ),
        migrations.DeleteModel(
            name='FieldValueThrough',
        ),
        migrations.AddField(
            model_name='fieldoptionthrough',
            name='field',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='field_options', to='django_form_generator.field', verbose_name='Field'),
        ),
        migrations.AddField(
            model_name='fieldoptionthrough',
            name='option',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='field_options', to='django_form_generator.option', verbose_name='Option'),
        ),
        migrations.AddField(
            model_name='field',
            name='options',
            field=models.ManyToManyField(help_text='Only for multi value fields like Dropdown, Radio, Checkbox, etc...', related_name='fields', through='django_form_generator.FieldOptionThrough', to='django_form_generator.option', verbose_name='Options'),
        ),
        migrations.AddIndex(
            model_name='fieldoptionthrough',
            index=models.Index(fields=['weight'], name='f_g_fieldoptionthrough_weight'),
        ),
    ]



from django.core.validators import FileExtensionValidator
from django.utils.translation import gettext as _
from rest_framework import serializers
from django_form_generator.common.utils import FileSizeValidator
from django_form_generator.models import Form, Option, Field
from django_form_generator.settings import form_generator_settings as fg_settings
from drf_recaptcha.fields import ReCaptchaV3Field

class CustomMultipleChoiceField(serializers.MultipleChoiceField):

    def to_internal_value(self, data):
        return list(super().to_internal_value(data))


class FormSerializer(serializers.ModelSerializer):

    class Meta:
        model = Form
        fields = ['id', 'title', 'slug', 'status']


class FormFullSerializer(serializers.ModelSerializer):
    form_fields = serializers.SerializerMethodField()
    
    class Meta:
        model = Form
        fields = ['title', 'slug', 'status', 'submit_text', 'redirect_url', 'success_message', 'style', 'direction', 'limit_to', 'valid_from', 'valid_to', 'is_editable', 'form_fields']

    def get_form_fields(self, obj):
        return obj.render_fields


class BaseFormSerializer(serializers.Serializer):

    def __init__(self, instance=None, data=None, **kwargs):
        super().__init__(instance, data, **kwargs)
        self._initial_fields()


    def _initial_fields(self):
        if self.form:
            for field in self.form.get_fields():
                method = f"prepare_{field.genre}"
                if hasattr(self, method):
                    self.fields[field.name] = getattr(self, method)(field)
                self._handel_required_fields(field, self.fields[field.name])

    def _handel_required_fields(self, field: Field, form_field):
        if self.initial_data and field.content_object:
            if isinstance(field.content_object, Option):
                field_name = self.form.get_fields(extra={"id__in":field.content_object.fields.values_list('id', flat=True)}).last().name
                parent_data = self.initial_data.get(field_name, '')
                if (isinstance(parent_data, list) and field.object_id not in parent_data) or (field.object_id == parent_data):
                    form_field.allow_null = True
                else:
                    form_field.read_only = False
            else:
                parent_data = self.initial_data.get(field.content_object.name, '')
                if len(parent_data) <= 0:
                    form_field.allow_null = True
                else:
                    form_field.read_only = False
    
    def prepare_text_input(self, field: Field):
        field_attrs: dict = field.build_serializer_attrs()
        return serializers.CharField(**field_attrs)

    def prepare_text_area(self, field: Field):
        field_attrs: dict = field.build_serializer_attrs()
        return serializers.CharField(**field_attrs)

    def prepare_number(self, field: Field):
        field_attrs: dict = field.build_serializer_attrs()
        return serializers.IntegerField(**field_attrs)

    def prepare_date(self, field: Field):
        field_attrs: dict = field.build_serializer_attrs()
        return serializers.DateField(**field_attrs)

    def prepare_time(self, field: Field):
        field_attrs: dict = field.build_serializer_attrs()
        return serializers.TimeField(**field_attrs)

    def prepare_datetime(self, field: Field):
        field_attrs: dict = field.build_serializer_attrs()
        return serializers.DateTimeField(**field_attrs)

    def prepare_email(self, field: Field):
        field_attrs: dict = field.build_serializer_attrs()
        return serializers.EmailField(**field_attrs)

    def prepare_password(self, field: Field):
        field_attrs: dict = field.build_serializer_attrs()
        return serializers.CharField(**field_attrs)

    def prepare_checkbox(self, field: Field):
        field_attrs: dict = field.build_serializer_attrs()
        return serializers.BooleanField(**field_attrs)

    def prepare_dropdown(self, field: Field):
        choices = field.get_choices().values_list("id", "name")
        field_attrs: dict = field.build_serializer_attrs()
        return serializers.ChoiceField(choices=choices, **field_attrs)

    def prepare_multi_checkbox(self, field: Field):
        choices = field.get_choices().values_list("id", "name")
        field_attrs: dict = field.build_serializer_attrs()
        return CustomMultipleChoiceField(choices=choices, **field_attrs)

    def prepare_radio(self, field: Field):
        choices = field.get_choices().values_list("id", "name")
        field_attrs: dict = field.build_serializer_attrs()
        return serializers.ChoiceField(choices=choices, **field_attrs)

    def prepare_hidden(self, field: Field):
        field_attrs: dict = field.build_serializer_attrs()
        return serializers.HiddenField(**field_attrs)

    def prepare_captcha(self, field: Field):
        field_attrs: dict = field.build_serializer_attrs()
        return ReCaptchaV3Field(**field_attrs)

    def prepare_upload_file(self, field: Field):
        message = (
            _("The file size is more than limit (limited size: %s bytes)")
            % field.file_size
        )
        field_attrs: dict = field.build_serializer_attrs()
        validators = [FileSizeValidator(field.file_size, message),
                    FileExtensionValidator(field.get_file_types()),]
        return serializers.FileField(validators=validators, **field_attrs)


class FormGeneratorSerializer(BaseFormSerializer):

    # form = serializers.SerializerMethodField()

    # def get_form(self, obj):
    #     return FormFullSerializer(self.form)

    def __init__(self, instance=None, data=None, *args, **kwargs):
        serializers.Serializer.__init__(self, instance, data, *args, **kwargs)
        self.form = self.context.get('form')
        self.request = self.context.get('request')
        self.user_ip = self.context.get('user_ip')
        self._initial_fields()


    def save(self):
        form_data = self.validated_data
        form_data.setdefault("request", self.request)
        save_module = fg_settings.FORM_RESPONSE_SAVE  
        save_module(self.form, form_data, self.user_ip)# type: ignore

class FormGeneratorResponseSerializer(BaseFormSerializer):
    def __init__(self, instance=None, data=None, *args, **kwargs):
        serializers.Serializer.__init__(self, instance, data, *args, **kwargs)
        self.form = self.context['form']
        self.request = self.context['request']
        self.form_response = self.context['form_response']
        self._initial_fields()

    @property
    def output_data(self):
        fields = self.form.render_fields.copy()
        for i, field in enumerate(fields):
            if field['attrs']['write_only']:
                fields.pop(i)
                i -= 1
            field.update({'value': self.form_response.get_data()[i]['value']})
        return fields

    def _initial_fields(self):
        form_response_data = self.form_response.get_data()
        for i, field in enumerate(self.form.get_fields()):
            field_name = field.name
            method = f"prepare_{field.genre}"
            if hasattr(self, method):
                self.fields[field_name] = getattr(self, method)(field)
                if not self.form.is_editable:
                    self.fields[field_name].read_only = True
                try:
                    initial_value = form_response_data[i].get(
                        "value", None
                    )
                    self.fields[field_name].initial = initial_value
                    if initial_value:
                        try:
                            self.fields[field_name].read_only = False
                        except KeyError:
                            pass
                except IndexError:
                    pass
                
                self._handel_required_fields(field, self.fields[field.name])

    def save(self, **kwargs):
        save_module = fg_settings.FORM_RESPONSE_SAVE
        form_data = self.form_response.pure_data
        for changed_data in self.validated_data.keys():
            form_data[changed_data] = self.validated_data[changed_data]

        form_data.setdefault("request", self.request)
        return save_module(self.form, form_data, update_form_response_id=self.form_response.id)# type: ignore

